// Copyright (c) 1995, 1996 Regents of the University of California.
// All rights reserved.
//
// This software was developed by the Arcadia project
// at the University of California, Irvine.
//
// Redistribution and use in source and binary forms are permitted
// provided that the above copyright notice and this paragraph are
// duplicated in all such forms and that any documentation,
// advertising materials, and other materials related to such
// distribution and use acknowledge that the software was developed
// by the University of California, Irvine.  The name of the
// University may not be used to endorse or promote products derived
// from this software without specific prior written permission.
// THIS SOFTWARE IS PROVIDED ``AS IS'' AND WITHOUT ANY EXPRESS OR
// IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
// WARRANTIES OF MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE.

// File: FigRect.java
// Classes: FigRect
// Original Author: ics125b spring 1996
// $Id: FigRect.java,v 1.4 1996/09/11 02:05:17 jrobbins Exp $

package uci.graphedit;


import java.applet.*;
import java.awt.*;
import java.io.*;
import java.util.*;

/** Primitive Fig to draw rectangles on a LayerDiagram */

public class FigRect extends Fig {

  /* number of handles on the boundary rectangle */
  public static final int numHandles = 8;

  /** Construct a new FigRect w/ the given position, size, and line color */
  public FigRect(int x, int y, int r_width, int r_height, Color r_color){
    super(x, y, r_width, r_height, r_color, null, numHandles);
  }

  /** Construct a new FigRect w/ the given position, size, and attrobutes */
  public FigRect(int x, int y, int r_width, int r_height, Hashtable gAttrs){
    super(x, y, r_width, r_height, Color.black, null, numHandles);
    setGraphicAttributes(gAttrs);
  }

  /** Construct a new FigRect w/ the given position, size, line color,
   * and fill color*/
  public FigRect(int x, int y, int r_width, int r_height,
		 Color r_color, Color f_color) {
    super(x, y, r_width, r_height, r_color, f_color, numHandles);
  }


  /** Draw this FigRect */
  public void draw(Graphics g) {
    int rx = position().x;
    int ry = position().y;
    int new_x, new_y, new_w, new_h;

    /* Compensate for negative width and height (for drag) */
    if (objectWidth > 0) { new_x = rx; new_w = objectWidth; }
    else { new_x = rx + objectWidth; new_w = -objectWidth; }
    
    if (objectHeight > 0) { new_y = ry; new_h = objectHeight; }
    else { new_y = ry + objectHeight; new_h = -objectHeight; }
    
    if (objectFillColor != null) {
      g.setColor(objectFillColor);
      g.fillRect(new_x, new_y, new_w, new_h);
    }
    g.setColor(objectLineColor);
    g.drawRect(new_x, new_y, new_w, new_h);
  }



  /** Position and draw handles on this FigRect */
  public void drawSelected(Graphics g) {
    int rx = position().x;
    int ry = position().y;
    int new_x, new_y, new_w, new_h;

    /* Compensate for negative width and height (for drag) */
    if (objectWidth > 0) { new_x = rx; new_w = objectWidth; }
    else { new_x = rx + objectWidth; new_w = -objectWidth; }
    
    if (objectHeight>0) { new_y = ry; new_h = objectHeight; }
    else { new_y = ry + objectHeight; new_h = -objectHeight; }
    
    /* Set the handle point positions*/
    _handleRects[0].move(new_x - HAND_SIZE/2,
			  new_y - HAND_SIZE/2);
    _handleRects[1].move(new_x + (new_w/2) - HAND_SIZE/2,
			  new_y - HAND_SIZE/2);
    _handleRects[2].move(new_x + (new_w) - HAND_SIZE/2,
			  new_y - HAND_SIZE/2);
    _handleRects[3].move(new_x + (new_w) - HAND_SIZE/2,
			  new_y + (new_h/2) - HAND_SIZE/2);
    _handleRects[4].move(new_x + (new_w) - HAND_SIZE/2,
			  new_y + new_h - HAND_SIZE/2);
    _handleRects[5].move(new_x + (new_w/2) - HAND_SIZE/2,
			  new_y + new_h - HAND_SIZE/2);
    _handleRects[6].move(new_x - HAND_SIZE/2,
			  new_y + new_h - HAND_SIZE/2);
    _handleRects[7].move(new_x - HAND_SIZE/2,
			  new_y + (new_h/2) - HAND_SIZE/2);
    drawHandles(g);
  }

  /** When this FigRect is selected in an Editor, use SelectionHandles
   * to record that fact and process events */
  public Selection selectionObject() { return new SelectionHandles(this); }

  /** Reply true if the given mouse coordinates are inside or "near"
   * this FigRect. Needs-More-Work: I whould have separate near() and
   * inside() functions. */
  public boolean inside(int x, int y) {
    Rectangle rect1, rect2;
    Point p = position();
    /* Set the bounding rectangle*/
    rect1 = new Rectangle(p.x - GRIP_MARGIN,
			  p.y - GRIP_MARGIN,
			  objectWidth  + GRIP_MARGIN * 2,
			  objectHeight + GRIP_MARGIN * 2);
    rect2 = new Rectangle(p.x + GRIP_MARGIN,
			  p.y + GRIP_MARGIN,
			  objectWidth - GRIP_MARGIN * 2,
			  objectHeight - GRIP_MARGIN * 2);

    if (objectFillColor != null) return rect1.inside(x,y);
    else return rect1.inside(x,y) && !rect2.inside(x,y);
  }


  /** When the user drags a handle, update the position or size of
   * this FigRect as needed. */
  public void dragHandle(int mouse_x,int mouse_y,
                         int anchor_x,int anchor_y,
                         int handle ) {
    Point p = position();
    int old_tlx = p.x;		// old top-left corner
    int old_tly = p.y;
    int old_brx = p.x + objectWidth; // old bottom-right corner
    int old_bry = p.y + objectHeight;

    //startTrans();
    switch (handle) {
    case 0:	/* The top left handle */
      p.x = mouse_x;
      p.y = mouse_y;
      objectWidth = old_brx - mouse_x;
      objectHeight = old_bry - mouse_y;
      break;
    case 1:	/* The top middle handle */
      p.y = mouse_y;
      objectHeight = old_bry - mouse_y;
      break;
    case 2:   /* The top right handle */
      p.y = mouse_y;
      objectWidth = mouse_x - old_tlx;
      objectHeight = old_bry - mouse_y;
      break;
    case 3:   /* The right middle handle */
      objectWidth = mouse_x - old_tlx;
      break;
    case 4:   /* The right bottom handle */
      objectWidth = mouse_x - old_tlx;
      objectHeight = mouse_y - old_tly;
      break;
    case 5:	/* The bottom middle handle */
      objectHeight = mouse_y - old_tly;
      break;
    case 6:	/* The bottom left handle */
      p.x = mouse_x;
      objectHeight = mouse_y - old_tly;
      objectWidth = old_brx - mouse_x;
      break;
    case 7:
      p.x = mouse_x;
      objectWidth = old_brx - mouse_x;
      break;
    }
    if (objectWidth < MIN_SIZE) {
      objectWidth = MIN_SIZE;
      if (handle == 0 || handle == 6 || handle == 7)
	p.x = old_brx - MIN_SIZE;
    }
    if (objectHeight < MIN_SIZE) {
      objectHeight = MIN_SIZE;
      if (handle == 0 || handle == 1 || handle==2)
	p.y = old_bry - MIN_SIZE;
    }
  } /* end method dragHandle */


} /* end of FigRect class */




